/**

BB-Flat

https://www.mql5.com/en/forum/88058

This is a lagging indicator. At each point in the MODE_MAIN moving average of the BB, a best fit
line is calculated from "lookback" prior values. You can shorten the lookback variable to reduce
lag, but this increases the noise (more dots shown).

Note: The lookback must be less than the BB moving average period.

Definition of Flat = 0 slope, plain and simple.

So, this indicator first calculates the slope of a best fit line through the past "lookback" values
using least square method. Since an exactly 0 slope is unlikely, a tolerance value is introduced.
Increasing the slope tolerance show more dots.

The standard deviation is calculated upon the distance from the points to the calculated best fit
line. This value discerns how good the "fit" to the line was. Decreasing this number eliminates
noise at the expense of removing dots.

Finally, the InpAlmostPercent variable is a percent (e.g. 0.10 = 10%) more than each tolerance
value. This shows dots in a different color. Set to 0 to disable.
*/

#property copyright "2018 Anthony J. Garot"
#property link      "http://www.garot.com/trading"

//---- indicator settings
//#property indicator_chart_window
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_plots   1
#property indicator_minimum .99999
#property indicator_maximum 1.00001
#property indicator_type1   DRAW_COLOR_ARROW
#property indicator_color1  clrDarkSlateGray,clrPaleGreen,clrYellow
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
#property indicator_label1  "BB-Flat"

#include <Math\\Stat\\Math.mqh>

//---- input parameters
// Go with same period as BB
input int               InpBandsPeriod=20;							// BB Period
input int				InpLookback=10 ;							// Lookback
input double			InpStdDevTolerance=0.02;					// STDEV Tolerance
input double			InpSlopeTolerance=0.01;						// Slope Tolerance
input double  			InpAlmostPercent=0.05;						// % For "Almost Flat"
// https://www.mql5.com/en/docs/constants/objectconstants/wingdings
// 108 is filled circle
// 161 is empty circle
// 174 is a star
// 240 is a right arrow, points to open of next bar
// 232 is filled right arrow
input ushort InpWingdingsCode = 108;

//---- indicator buffers
double ExtArrowBuffer[];			// Where the arrow goes
double ExtArrowColors[];			// Color of the arrow
double ExtBandsMain[];

//---- handles for indicators
int    ExtBandsHandle;

// What's our minimum number of bars to perform a calculation?
int ExtBarsMinimum = InpBandsPeriod + 1;		// The additional 1 is because of smoothing value

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
	if ( InpLookback > InpBandsPeriod )
	{
		Print("Lookback must be less than BB Period");
		return(INIT_PARAMETERS_INCORRECT);
	}

	//---- indicator buffers mapping
	SetIndexBuffer(0,ExtArrowBuffer,INDICATOR_DATA);
	SetIndexBuffer(1,ExtArrowColors,INDICATOR_COLOR_INDEX);
	SetIndexBuffer(2,ExtBandsMain,INDICATOR_DATA);

	// set accuracy
	IndicatorSetInteger(INDICATOR_DIGITS,_Digits);

	// PLOT Arrow values
	PlotIndexSetInteger(0,PLOT_ARROW,InpWingdingsCode);		// Define the symbol InpWingdingsCode for drawing in PLOT_ARROW
	PlotIndexSetInteger(0,PLOT_ARROW_SHIFT,10);	// Set the vertical shift (down) of arrows in pixels
	PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,0);	// Set as an empty value 0

	// get handles for the indicators used in this indicator
	ExtBandsHandle=iBands(NULL, 0, InpBandsPeriod,0, MODE_EMA, PRICE_HIGH);

	// initialization done
	return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//|  Alligator OnCalculate function                                  |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
				const int prev_calculated,
				const datetime &time[],
				const double &open[],
				const double &high[],
				const double &low[],
				const double &close[],
				const long &tick_volume[],
				const long &volume[],
				const int &spread[])
{
	// check for rates total
	if(rates_total<ExtBarsMinimum)
		return(0); // not enough bars for calculation

	// not all data may be calculated
	int calculated=BarsCalculated(ExtBandsHandle);
	if(calculated<rates_total)
	{
		Print("Not all data of ExtBandsHandle is calculated (",calculated,"bars ). Error",GetLastError());
		return(0);
	}

	// we don't have to copy all of the data all the time
	int to_copy;
	if(prev_calculated>rates_total || prev_calculated<0)
	{
		to_copy=rates_total;
	}
	else
	{
		to_copy=rates_total-prev_calculated;
		if(prev_calculated>0) to_copy++;
	}

	//---- get buffers
	if(IsStopped()) return(0); //Checking for stop flag
	if(CopyBuffer(ExtBandsHandle,0,0,to_copy,ExtBandsMain)<=0)
	{
		Print("Getting ExtBandsHandle is failed! Error",GetLastError());
		return(0);
	}

	// correct position, when it's first iteration
	int pos = prev_calculated - 1;
	if ( pos <= 0 )
	{
		pos = 1;
		ExtArrowBuffer[0] = 0.;
	}

	// main cycle
	// loop over all bars
	for(int i=pos+ExtBarsMinimum;i<rates_total && !IsStopped();i++)
	{
		ExtArrowBuffer[i]=1;
		//ExtArrowColors[i] = isFlat(i, ExtBandsMain, InpLookback, i > rates_total - 80 && i < rates_total - 1);		// debugging
		ExtArrowColors[i] = isFlat(	i,
									ExtBandsMain,
									InpLookback,
									InpAlmostPercent,
									InpSlopeTolerance,
									InpStdDevTolerance
									);
	}

	// return value of prev_calculated for next call
	return(rates_total);
}

// Uses least squares fit to determine if flat
int isFlat(	int iBar,
			double &yValues[],
			int lookback,
			double almostPercent,
			double slopeTolerance,
			double stdDevTolerance,
			bool debug = false)
{
	// Get the sums
	double ySum = 0.;
	double xSum = 0.;
	for ( int i = iBar; i > iBar - lookback; i-- )
	{
		ySum += yValues[i];
		xSum += iBar - i;

		if ( debug )
		{
			Print(StringFormat(	"HEY: iBar [%d] i [%d] lookback [%d] xSum [%.2f] ySum [%.2f]",
								iBar,
								i,
								lookback,
								xSum,
								ySum
								));
		}
	}

	// xMean and yMean
	double xMean = xSum / lookback;
	double yMean = ySum / lookback;

	if ( debug )
	{
		Print(StringFormat(	"HEY: lookback [%d] ySum [%.2f] xMean [%.2f] yMean [%.2f]",
							lookback,
							ySum,
							xMean,
							yMean
							));
	}

	// numerator and denominator
	double numerator = 0.0;
	double denominator = 0.0;
	for ( int i = iBar; i > iBar - lookback; i-- )
	{
		numerator	+= ( (iBar - i) - xMean) * ( yValues[i] - yMean );
		denominator += MathSqr( (iBar - i) - xMean );
	}

	if ( MathAbs(denominator)<1.0E-10 )
	{
		Print("Division by zero imminent.");
		return false;
	}

	double slope = numerator / denominator;

	// Determine distance of points to the line created.
	// This is simply y = mx + b
	// The y intercept (b) probably doesn't matter.
	double distances[];
	ArrayResize(distances, lookback);
	for ( int i = iBar; i > iBar - lookback; i-- )
	{
		//distances[(iBar - i)] = yValues[i] - slope * (iBar - i);
		distances[(iBar - i)] = MathAbs(slope * (iBar - i) - yValues[i]) / MathSqrt(1. + MathSqr(slope));
	}

	double stdev = MathStandardDeviation(distances);

	if ( debug )
	{
		Print(StringFormat(	"HEY: [%d] slope [%.5f] stdev [%.3f] xMean [%.2f] yMean [%.2f] numerator [%.5f] denominator [%.5f]",
							iBar,
							slope,
							stdev,
							xMean,
							yMean,
							numerator,
							denominator
							));
	}

	double almostMultiplier = 1. + almostPercent;

	if ( MathAbs(slope) < slopeTolerance && stdev < stdDevTolerance )
	{
		return 1;
	}
	else if ( almostPercent > 0 && MathAbs(slope) < slopeTolerance * almostMultiplier && stdev < stdDevTolerance * almostMultiplier )
	{
		return 2;
	}
	else
	{
		return 0;
	}
}

// Generic square function
template<typename T>
T MathSqr(T value)
{
	return (value * value);
}
